/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.smart.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayu.daf.core.boot.ctrl.DafController;
import com.dayu.daf.core.log.annotation.ApiLog;
import com.dayu.daf.core.log.exception.ServiceException;
import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.apache.commons.codec.Charsets;
import org.energy.modules.auth.utils.TokenUtil;
import org.energy.modules.smart.entity.WorkOrder;
import org.energy.modules.smart.excel.WorkOrderExcel;
import org.energy.modules.smart.service.IWorkOrderService;
import org.energy.modules.smart.vo.WorkOrderVO;
import org.energy.modules.system.util.DataUtils;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 *  控制器
 *
 * @author Daf
 * @since 2024-07-12
 */
@RestController
@AllArgsConstructor
@RequestMapping("/workorder")
@Api(value = "", tags = "接口")
public class WorkOrderController extends DafController {

	private IWorkOrderService workOrderService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入workOrder")
	public R<WorkOrderVO> detail(WorkOrder workOrder) {
		WorkOrderVO detail = workOrderService.selectDetail(workOrder.getId());
		return R.data(detail);
	}

	/**
	 * 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入workOrder")
	public R<IPage<WorkOrder>> list(WorkOrder workOrder, Query query) {
		IPage<WorkOrder> pages = workOrderService.page(Condition.getPage(query), Condition.getQueryWrapper(workOrder));
		return R.data(pages);
	}

	/**
	 * 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入workOrder")
	public R<IPage<WorkOrderVO>> page(WorkOrderVO workOrderVO, Query query) {
		IPage<WorkOrderVO> pages = workOrderService.selectWorkOrderPage(Condition.getPage(query), workOrderVO);
		return R.data(pages);
	}

	/**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入workOrder")
	public R save(@Valid @RequestBody WorkOrder workOrder) {
		return R.status(workOrderService.save(workOrder));
	}

	/**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入workOrder")
	public R update(@Valid @RequestBody WorkOrder workOrder) {
		return R.status(workOrderService.updateById(workOrder));
	}

	/**
	 * 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入workOrder")
	public R submit(@Valid @RequestBody WorkOrder workOrder) {
		return R.status(workOrderService.saveOrUpdate(workOrder));
	}


	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(workOrderService.removeByIds(Func.toLongList(ids)));
	}

	/**
	 * 审核
	 */
	@PostMapping("/auditing")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R auditing(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.approve(2,ids);
	}

	/**
	 * 驳回
	 */
	@PostMapping("/reject")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R reject(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.approve(3,ids);
	}

	/**
	 * 驳回OR审核
	 */
	public R approve(Integer status, String ids) {
		List<WorkOrder> list = new ArrayList<>();
		for (String id : ids.split(",")) {
			WorkOrder workOrder = workOrderService.getById(Long.parseLong(id));
			if (workOrder == null) {
				throw new ServiceException(TokenUtil.SYSTEM_NOT_DATA);
			}
			workOrder.setId(Long.parseLong(id));
			workOrder.setApprovalStatus(status);
			list.add(workOrder);
		}
		return R.status(workOrderService.updateBatchById(list));
	}

	/**
	 * 导出
	 */
	@SneakyThrows
	@GetMapping("export")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "导出", notes = "传入")
	@ApiLog
	public void exportUser(@ApiIgnore @RequestParam Map<String, Object> entity, HttpServletResponse response) {
		entity.remove("daf-auth");
		WorkOrderVO vo = DataUtils.mapToEntity(entity, WorkOrderVO::new);
		List<WorkOrderExcel> list = workOrderService.export(vo);
		response.setContentType("application/vnd.ms-excel");
		response.setCharacterEncoding(Charsets.UTF_8.name());
		String fileName = URLEncoder.encode("工单数据导出", Charsets.UTF_8.name());
		response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
		EasyExcel.write(response.getOutputStream(), WorkOrderExcel.class).sheet("工单").doWrite(list);
	}

}
