/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.spares.controller;

import com.xkcoding.http.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import org.energy.modules.spares.service.IMaterialService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.energy.modules.spares.entity.Outbound;
import org.energy.modules.spares.vo.OutboundVO;
import org.energy.modules.spares.service.IOutboundService;
import com.dayu.daf.core.boot.ctrl.DafController;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 出库 控制器
 *
 * @author Daf
 * @since 2024-07-15
 */
@RestController
@AllArgsConstructor
@RequestMapping("/spares/outbound")
@Api(value = "出库", tags = "出库接口")
public class OutboundController extends DafController {

	private IOutboundService outboundService;
	private IMaterialService materialService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入outbound")
	public R<Outbound> detail(Outbound outbound) {
		Outbound detail = outboundService.getOne(Condition.getQueryWrapper(outbound));
		return R.data(detail);
	}

	/**
	 * 分页 出库
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入outbound")
	public R<IPage<Outbound>> list(Outbound outbound, Query query) {
		IPage<Outbound> pages = outboundService.page(Condition.getPage(query), Condition.getQueryWrapper(outbound));
		return R.data(pages);
	}

	/**
	 * 自定义分页 出库
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入outbound")
	public R<IPage<OutboundVO>> page(OutboundVO outbound, Query query) {
		IPage<OutboundVO> pages = outboundService.selectOutboundPage(Condition.getPage(query), outbound);
		return R.data(pages);
	}

	/**
	 * 新增 出库
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入outbound")
	public R save(@Valid @RequestBody Outbound outbound) {
		return R.status(outboundService.save(outbound));
	}

	/**
	 * 修改 出库
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入outbound")
	public R update(@Valid @RequestBody Outbound outbound) {
		if (outbound.getIstatus() == 2)
		{
			materialService.updateNum(outbound.getMaterialNo(), Math.negateExact(outbound.getOutboundCount()));
		}
		return R.status(outboundService.updateById(outbound));
	}

	/**
	 * 新增或修改 出库
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入outbound")
	public R submit(@Valid @RequestBody Outbound outbound) {
		String maxNo = outboundService.getMaxNo("'%" + outbound.getMaterialNo() + "%'");
		int number;
		if (StringUtil.isNotEmpty(maxNo)){
			number = Integer.parseInt(maxNo.substring(11));
			number++;
		}else{
			number = 1;
		}
		String numFormat = String.format("%06d", number);
		String no = outbound.getMaterialNo() + "OUT" + numFormat;
		outbound.setOutboundNo(no);
		Date date = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		String time = sdf.format(date);
		outbound.setOutboundTime(time);
		outbound.setIstatus(1L);
		return R.status(outboundService.saveOrUpdate(outbound));
	}


	/**
	 * 删除 出库
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(outboundService.deleteLogic(Func.toLongList(ids)));
	}

	/**
	 * 列表 出库详细
	 */
	@GetMapping("/getList")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "列表", notes = "传入outbound")
	public R<List<Outbound>> getList(Outbound outbound) {
		List<Outbound> list = outboundService.list(Condition.getQueryWrapper(outbound));
		return R.data(list);
	}


}
