/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE 3.0;
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.gnu.org/licenses/lgpl.html
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.spares.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dayu.daf.core.log.annotation.ApiLog;
import com.dayu.daf.core.tool.constant.DafConstant;
import io.micrometer.core.instrument.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.mp.support.Query;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.utils.Func;
import lombok.SneakyThrows;
import org.apache.commons.codec.Charsets;
import org.energy.modules.spares.excel.WarehouseExcel;
import org.energy.modules.system.util.DataUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.energy.modules.spares.entity.Warehouse;
import org.energy.modules.spares.vo.WarehouseVO;
import org.energy.modules.spares.service.IWarehouseService;
import com.dayu.daf.core.boot.ctrl.DafController;
import springfox.documentation.annotations.ApiIgnore;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * spt_warehouse 控制器
 *
 * @author Daf
 * @since 2024-07-15
 */
@RestController
@AllArgsConstructor
@RequestMapping("/warehouse")
@Api(value = "spt_warehouse", tags = "spt_warehouse接口")
public class WarehouseController extends DafController {

	private IWarehouseService warehouseService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入warehouse")
	public R<Warehouse> detail(Warehouse warehouse) {
		Warehouse detail = warehouseService.getOne(Condition.getQueryWrapper(warehouse));
		return R.data(detail);
	}

	/**
	 * 分页 spt_warehouse
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入warehouse")
	public R<IPage<Warehouse>> list(Warehouse warehouse, Query query) {
		QueryWrapper<Warehouse> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(warehouse.getCode())) {
			qw.lambda().like(Warehouse::getCode, warehouse.getCode());
		}
		if (StringUtils.isNotEmpty(warehouse.getName())) {
			qw.lambda().like(Warehouse::getName, warehouse.getName());
		}
		if (StringUtils.isNotEmpty(warehouse.getAddress())) {
			qw.lambda().like(Warehouse::getAddress, warehouse.getAddress());
		}
		if (warehouse.getType() != null) {
			qw.lambda().eq(Warehouse::getType, warehouse.getType());
		}
		qw.lambda().eq(Warehouse::getIsDeleted, DafConstant.DB_NOT_DELETED);
		qw.lambda().orderByDesc(Warehouse::getUpdateTime);
		IPage<Warehouse> pages = warehouseService.page(Condition.getPage(query), qw);
		return R.data(pages);
	}

	/**
	 * 自定义分页 spt_warehouse
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入warehouse")
	public R<IPage<WarehouseVO>> page(WarehouseVO warehouse, Query query) {
		IPage<WarehouseVO> pages = warehouseService.selectWarehousePage(Condition.getPage(query), warehouse);
		return R.data(pages);
	}

	/**
	 * 新增 spt_warehouse
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入warehouse")
	public R save(@Valid @RequestBody Warehouse warehouse) {
		return R.status(warehouseService.save(warehouse));
	}

	/**
	 * 修改 spt_warehouse
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入warehouse")
	public R update(@Valid @RequestBody Warehouse warehouse) {
		return R.status(warehouseService.updateById(warehouse));
	}

	/**
	 * 新增或修改 spt_warehouse
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入warehouse")
	public R submit(@Valid @RequestBody Warehouse warehouse) {
		if (warehouse.getId() == null) {
			QueryWrapper<Warehouse> qw = new QueryWrapper<>();
			qw.lambda().select(Warehouse::getCode)
				.eq(Warehouse::getType, warehouse.getType())
				.eq(Warehouse::getIsDeleted, DafConstant.DB_NOT_DELETED)
				.orderByDesc(Warehouse::getCode)
				.last("LIMIT 1");
			String code = "";
			Warehouse warehouseSelect = warehouseService.getOne(qw);
			if (warehouseSelect != null) {
				String maxCode = warehouseSelect.getCode();
				if ("9999".equals(maxCode)) {
					return R.fail("操作失败，编号已越位");
				}
				code = "CK" + warehouse.getType() + DataUtils.incrementLast(maxCode, 4);
			} else {
				code = "CK" + warehouse.getType() + "0001";
			}
			warehouse.setCode(code);
		}
		return R.status(warehouseService.saveOrUpdate(warehouse));
	}


	/**
	 * 删除 spt_warehouse
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(warehouseService.deleteLogic(Func.toLongList(ids)));
	}

	/**
	 * 审核
	 */
	@PostMapping("/auditing")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R auditing(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.approve(2,ids);
	}

	/**
	 * 驳回
	 */
	@PostMapping("/reject")
	@ApiOperationSupport(order = 9)
	@ApiOperation(value = "更新", notes = "传入ids")
	public R reject(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return this.approve(3,ids);
	}

	/**
	 * 驳回OR审核
	 */
	public R approve(Integer status, String ids) {
		List<Warehouse> list = new ArrayList<>();
		for (String id : ids.split(",")) {
			Warehouse warehouse = new Warehouse();
			warehouse.setId(Long.parseLong(id));
			warehouse.setApprovalStatus(status);
			list.add(warehouse);
		}
		return R.status(warehouseService.updateBatchById(list));
	}

	/**
	 * 导出
	 */
	@SneakyThrows
	@GetMapping("export")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "导出", notes = "传入")
	@ApiLog
	public void exportUser(@ApiIgnore @RequestParam Map<String, Object> entity, HttpServletResponse response) {
		entity.remove("daf-auth");
		WarehouseVO vo = DataUtils.mapToEntity(entity, WarehouseVO::new);
		List<WarehouseExcel> list = warehouseService.export(vo);
		response.setContentType("application/vnd.ms-excel");
		response.setCharacterEncoding(Charsets.UTF_8.name());
		String fileName = URLEncoder.encode("仓库数据导出", Charsets.UTF_8.name());
		response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
		EasyExcel.write(response.getOutputStream(), WarehouseExcel.class).sheet("仓库").doWrite(list);
	}

	/**
	 * 获取仓库信息
	 */
	@GetMapping("/getWarehouseList")
	@ApiOperationSupport(order = 11)
	@ApiOperation(value = "获取仓库信息", notes = "获取仓库信息")
	public R<List<Warehouse>> getWarehouseList() {
		List<Warehouse> noList = warehouseService.getWarehouseList();
		return R.data(noList);
	}

}
