/**
 * Copyright (c) 2018-2028, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.energy.modules.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dayu.daf.core.boot.ctrl.DafController;
import com.dayu.daf.core.log.annotation.ApiLog;
import com.dayu.daf.core.mp.support.Condition;
import com.dayu.daf.core.secure.DafUser;
import com.dayu.daf.core.tool.api.R;
import com.dayu.daf.core.tool.constant.DafConstant;
import com.dayu.daf.core.tool.node.INode;
import com.dayu.daf.core.tool.utils.CacheUtil;
import com.dayu.daf.core.tool.utils.Func;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.energy.modules.system.entity.Role;
import org.energy.modules.system.service.IRoleDataService;
import org.energy.modules.system.service.IRoleService;
import org.energy.modules.system.vo.GrantVO;
import org.energy.modules.system.vo.HomePageVO;
import org.energy.modules.system.vo.RoleVO;
import org.energy.modules.system.wrapper.RoleWrapper;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

import static com.dayu.daf.core.tool.utils.CacheUtil.SYS_CACHE;

/**
 * 控制器
 *
 * @author Chill
 */
@ApiIgnore
@RestController
@AllArgsConstructor
@RequestMapping("/daf-system/role")
@Api(value = "角色", tags = "角色")
public class RoleController extends DafController {

	private IRoleService roleService;

	private IRoleDataService roleDataService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入role")
	@ApiLog
	public R<RoleVO> detail(Role role) {
		Role detail = roleService.getOne(Condition.getQueryWrapper(role));
		return R.data(RoleWrapper.build().entityVO(detail));
	}

	/**
	 * 列表
	 */
	@GetMapping("/list")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleName", value = "参数名称", paramType = "query", dataType = "string"),
		@ApiImplicitParam(name = "roleAlias", value = "角色别名", paramType = "query", dataType = "string")
	})
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "列表", notes = "传入role")
	@ApiLog
	public R<List<INode>> list(@ApiIgnore @RequestParam Map<String, Object> role, DafUser dafUser) {
		QueryWrapper<Role> queryWrapper = Condition.getQueryWrapper(role, Role.class);
		List<Role> list = roleService.list((!dafUser.getTenantId().equals(DafConstant.ADMIN_TENANT_ID)) ? queryWrapper.lambda().eq(Role::getTenantId, dafUser.getTenantId()) : queryWrapper);
		return R.data(RoleWrapper.build().listNodeVO(list));
	}

	/**
	 * 获取角色树形结构
	 */
	@GetMapping("/tree")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "树形结构", notes = "树形结构")
	@ApiLog
	public R<List<RoleVO>> tree(String tenantId, DafUser dafUser) {
		List<RoleVO> tree = roleService.tree(Func.toStr(tenantId, dafUser.getTenantId()));
		return R.data(tree);
	}

	/**
	 * 获取指定角色树形结构
	 */
	@GetMapping("/tree-by-id")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "树形结构", notes = "树形结构")
	@ApiLog
	public R<List<RoleVO>> treeById(Long roleId, DafUser dafUser) {
		Role role = roleService.getById(roleId);
		List<RoleVO> tree = roleService.tree(Func.notNull(role) ? role.getTenantId() : dafUser.getTenantId());
		return R.data(tree);
	}

	/**
	 * 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "新增或修改", notes = "传入role")
	@ApiLog
	public R submit(@Valid @RequestBody Role role, DafUser user) {
		CacheUtil.clear(SYS_CACHE);
		if (Func.isEmpty(role.getId())) {
			role.setTenantId(user.getTenantId());
		}
		if (Func.isEmpty(role.getParentId())) {
			role.setParentId(0L);
		}
		return R.status(roleService.saveOrUpdate(role));
	}

	/**
	 * 删除L
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "删除", notes = "传入ids")
	@ApiLog
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		CacheUtil.clear(SYS_CACHE);
		return R.status(roleService.removeByIds(Func.toLongList(ids)));
	}

	/**
	 * 设置菜单权限
	 */
	@PostMapping("/grant")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "权限设置", notes = "传入roleId集合以及menuId集合")
	@ApiLog
	public R grant(@RequestBody GrantVO grantVO) {
		CacheUtil.clear(SYS_CACHE);
		boolean temp = roleService.grant(grantVO.getRoleIds(), grantVO.getMenuIds(), grantVO.getDataScopeIds());
		return R.status(temp);
	}

	@GetMapping("/honePageData")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "首页数据", notes = "传入")
	public R<HomePageVO> honePageData() {
		return R.data(roleDataService.findCurrHomePageInfo());
	}

}
